
/**
 *	@file		glControl.d
 *	@version	v0.01
 *	@date		01.12.2004
 *  @author		Burton Radons
 *  @modified	Miguel Ferreira Simoes (kobold at netcabo dot pt)
 *        
 *  This code is a slightly modification of unDig's CanvasGL control:
 *		http://www.dsource.org/projects/undig/
 *
 *	This software is provided 'as-is', without any express or implied
 *	warranty. In no event will the authors be held liable for damages
 *	of any kind arising from the use of this software.
 *      
 *  Copyright 2004 Burton Radons
 */

module dfl.canvasGL;

private import std.string;

private import dfl.winapi;
private import dfl.control;
private import dfl.event;
private import dfl.application;
private import derelict.opengl.gl;

/** Example
 *
 *  int main (char [] [] args) {
 *
 *		dglLoad();
 *
 *		Application.run(new MainForm());
 *		return 1;
 *	}
 *  
 *	class MainForm: Form {
 *		CanvasGL canvas;
 *	
 *		this() {
 *			with(canvas = new CanvasGL(this, "")) {
 *				bounds = Rectangle(25, 25, 100, 100);
 *				paint ~= &paintEvent;
 *			}
 *		}
 *
 *		void paintEvent(Object sender, PaintEventArgs ea) {
 *			canvas.beginPaint();
 *				glClearColor(0.0, 0.0, 0.0, 0.0);
 *				glClear(GL_COLOR_BUFFER_BIT);
 *				glColor3f(1.0, 1.0, 1.0);
 *				glOrtho(-1.0, 1.0, -1.0, 1.0, -1.0, 1.0); 
 *				glBegin(GL_POLYGON);
 *					glVertex2f(-0.5, -0.5);
 *					glVertex2f(-0.5, 0.5);
 *					glVertex2f(0.5, 0.5);
 *					glVertex2f(0.5, -0.5);	
 *				glEnd();
 *			canvas.endPaint();
 *		}
 *	}
 */

class CanvasGL : Control {

	/* Protected Data */

	protected HDC _hdc;
	protected HGLRC _hglrc;
	protected PAINTSTRUCT _painter;
	protected static const char[] _classname = "CanvasGl";

	/* Constructors & Destructors */

	static this() {
		WNDCLASS windowclass;
		windowclass.style = CS_OWNDC;
		windowclass.lpfnWndProc = &dflWndProc;
		windowclass.cbClsExtra = 0;
		windowclass.cbWndExtra = 0;
		windowclass.hInstance = GetModuleHandleA(null);
		windowclass.hCursor = LoadCursorA(cast(HANDLE) 0, IDC_ARROW);
		windowclass.hbrBackground = cast(HBRUSH)(COLOR_WINDOW + 1);
		windowclass.lpszMenuName = null;
		windowclass.lpszClassName = toStringz(_classname);
		RegisterClassA (&windowclass);
	}

	public this(Control parent, char[] text) {
		super(parent, text);
		wstyle |= WS_VISIBLE | WS_CHILD | WS_CLIPCHILDREN | WS_CLIPSIBLINGS;
	}

	public ~this() {
		wglMakeCurrent(null, null);
		wglDeleteContext(_hglrc);
    }

	/* BeginPaint & EndPaint Methods */

	public void beginPaint() {
		BeginPaint(hwnd, &_painter);
		makeCurrent();
		glViewport(0, 0, width(), height());
	}

	public void endPaint() {
		glFlush();
		SwapBuffers(_hdc);
		ReleaseDC(hwnd, _hdc);
		EndPaint(hwnd, &_painter);
	}

	/* Overriding CreateHandle */

	protected override void createHandle() {
		createClassHandle(_classname);
		createContext();
		onHandleCreated(EventArgs.empty);
	}

	/* Protected OpenGl Methods */

	protected void makeCurrent() {
		if (!wglMakeCurrent(_hdc, _hglrc))
			throw new Exception("CanvasGL: wglMakeCurrent.");
	}

	protected void createContext() {
		_hdc = GetDC(hwnd); 	
       
		PIXELFORMATDESCRIPTOR info;
		info.nSize = info.sizeof;
		info.nVersion = 1;
		info.dwFlags = PFD_DRAW_TO_WINDOW
					| PFD_SUPPORT_OPENGL
					| PFD_DOUBLEBUFFER;
		info.iPixelType = PFD_TYPE_RGBA;
		info.cColorBits = 24;
		info.cAlphaBits = 8;

		info.cAccumBits = 0;
		info.cDepthBits = 32;
		info.cStencilBits = 8;
		info.cAuxBuffers = 0;
		info.iLayerType = PFD_MAIN_PLANE;

		int index;
		if ((index = ChoosePixelFormat(_hdc, &info)) == 0)
			throw new Exception("CanvasGL: ChoosePixelFormat.");

		if (!SetPixelFormat(_hdc, index, &info))
			throw new Exception("CanvasGL: SetPixelFormat.");

		if ((_hglrc = wglCreateContext(_hdc)) === null)
			throw new Exception("CanvasGL: wglCreateContext.");

		if (!wglMakeCurrent(_hdc, _hglrc))
			throw new Exception("CanvasGL: wglMakeCurrent.");
    
		glPixelStorei(GL_PACK_ALIGNMENT, 1);
		glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
		makeCurrent();
    }
   
}
